/*****************************************************************************/
// MainWindow
// Written by Hartmut Reh
//
// MainWindow.cpp
//
//
// Copyright (c) 2004 OpenBeOS Project
//
// MainWindow.cpp
// Generated by Interface Elements (Window v2.3) on Apr 11 2004
// This is a user written class and will not be overwritten.
//
/*****************************************************************************/

#include "MainWindow.h"
#include "TheMessages.h"
#include <USBKit.h>
#include <stdlib.h>
#include <Beep.h>




MainWindow :: MainWindow(void)
						: IEWindow("MainWindow")
{
	Lock();
		InitInterface();
		GetPrefs();
		
		fReceiving		= false;
		EnableAllElements(false);
	
		fTheRoster 	= new TheUSBRoster(this);
		fTheRoster -> DeviceData(fVendorID, fProductID, fSerialNumber);
		fTheRoster -> Start();
	
	Unlock();
	Show();
}



MainWindow :: ~MainWindow(void)
{
	if (fTheRoster != NULL) delete fTheRoster;	
}


bool MainWindow :: QuitRequested()
{
   be_app->PostMessage(B_QUIT_REQUESTED);
   SetPrefs();
   delete fPrefs;														// save tp references
   return(true);
}



// Handling of user interface and other events
void MainWindow :: MessageReceived(BMessage *message)
{

	switch(message->what)
	{
		case IE_MAINWINDOW_INFO :			// 'Info' is pressed...
				be_app->PostMessage(B_ABOUT_REQUESTED);
			break;

		case IE_MAINWINDOW_VENDORID:	// Enter is pressed in 'VendorID'...
			break;

		case IE_MAINWINDOW_VENDORID_MODIFICATION:	// 'VendorID' modified...
			{
				const char *text = fVendorIDControl-> Text();
				fVendorID = (uint32) atol(text);
			}
			break;

		case IE_MAINWINDOW_PRODUCTID:	// Enter is pressed in 'ProductID'...
			break;

		case IE_MAINWINDOW_PRODUCTID_MODIFICATION:	// 'ProductID' modified...
			{
				const char *text = fProductIDControl-> Text();
				fProductID = (uint32) atol(text);
			}
			break;

		case IE_MAINWINDOW_SERIALNUMBER:	// Enter is pressed in 'SerialNumber'...
			break;

		case IE_MAINWINDOW_SERIALNUMBER_MODIFICATION:	// 'SerialNumber' modified...
			{
				const char *text = fSerialNumberControl-> Text();
				strcpy(fSerialNumber, text);
			}
			break;

		case IE_MAINWINDOW_SEARCHBUTTON:	// 'SearchButton' is pressed...
			{
				if (fTheRoster != NULL) delete fTheRoster;
				fTheRoster 	= new TheUSBRoster(this);
				fTheRoster -> DeviceData(fVendorID, fProductID, fSerialNumber);
				fTheRoster -> Start();			
			}
			break;

		case IE_MAINWINDOW_STARTBUTTON:	// 'StartButton' is pressed...
			{				
				fReceiving = !fReceiving;
				if ( fReceiving )
				{
				 	fReadImmediate -> SetEnabled(false);
				 	fStartButton->SetLabel(" Stop Reading");
				 	fTheRoster ->fIOWarrior24 -> WarriorThread(fReadImmediate->Value());	// start Thread
				}
				else
				{
				 	fReadImmediate -> SetEnabled(true);
				 	fStartButton->SetLabel(" Start Reading");
				 	fTheRoster -> fIOWarrior24 -> StopReadData();								// stop Thread
				 	ClearDisplay();
				}				
			}
			break;


		case DEVICE_ADDED_MSG:	// Device added
			{
				USBDevice *dev = fTheRoster -> fIOWarrior24->GetDevice();
				const char* productString = dev->ProductString();	// ProductString = IO-Warrior24
				fDeviceString -> SetText(productString);
				EnableAllElements(true);
				fTheRoster ->fIOWarrior24 -> WriteAllPorts(0xFFFF);		// Write data to IO-Warrior input mode			
			}
			break;


		case DEVICE_REMOVED_MSG:	// Device removed
			{				
				fDeviceString -> SetText(" No IO-Warrior24 Present");
				EnableAllElements(false);
				fReceiving = false;
				fReadImmediate -> SetValue(B_CONTROL_OFF);
				ClearDisplay();	
			}
			break;


		// hier wird in die 16-bit I/O-Ports geschrieben
		case IE_MAINWINDOW_SP07:	// 'SP0_7' is pressed...
		case IE_MAINWINDOW_SP06:	// 'SP0_6' is pressed...
		case IE_MAINWINDOW_SP05:	// 'SP0_5' is pressed...
		case IE_MAINWINDOW_SP04:	// 'SP0_4' is pressed...
		case IE_MAINWINDOW_SP03:	// 'SP0_3' is pressed...
		case IE_MAINWINDOW_SP02:	// 'SP0_2' is pressed...
		case IE_MAINWINDOW_SP01:	// 'SP0_1' is pressed...
		case IE_MAINWINDOW_SP00:	// 'SP0_0' is pressed...
		// Set Port 0

		case IE_MAINWINDOW_SP17:	// 'SP1_7' is pressed...
		case IE_MAINWINDOW_SP16:	// 'SP1_6' is pressed...
		case IE_MAINWINDOW_SP15:	// 'SP1_5' is pressed...
		case IE_MAINWINDOW_SP14:	// 'SP1_4' is pressed...
		case IE_MAINWINDOW_SP13:	// 'SP1_3' is pressed...
		case IE_MAINWINDOW_SP12:	// 'SP1_2' is pressed...
		case IE_MAINWINDOW_SP11:	// 'SP1_1' is pressed...
		case IE_MAINWINDOW_SP10:	// 'SP1_0' is pressed...

		// Set Port 1
			{
				int i;
				uint8 	wert;
				uint16	allPorts;
				wert = 0;
				for (i=0; i<=7; i++)
					if ( (fSP0[i]->Value()) == B_CONTROL_ON ) wert= wert +(uint8)pow(2,i);
				allPorts = (int16)wert;
			
				wert = 0;
				for (i=0; i<=7; i++)
					if ( (fSP1[i]->Value()) == B_CONTROL_ON ) wert = wert +(uint8)pow(2,i);
				allPorts = allPorts + 256*(uint16)wert;
			
				fTheRoster -> fIOWarrior24 -> WriteAllPorts(~allPorts);		// write data to IO-Warrior			
			}
			break;

		case IE_MAINWINDOW_INOUTBUTTON:	// 'INOUTButton' is pressed...
			{
				ClearDisplay();
				if (fInOutButton->Value() == B_CONTROL_OFF)		// Output
				{
					for (int i=0; i<8; i++)
					{
						fSP0[i] -> SetEnabled(true);
						fSP1[i] -> SetEnabled(true);
					}
				}
				else
				{
					for (int i=0; i<8; i++)
					{
						fTheRoster -> fIOWarrior24 -> WriteAllPorts(0xFFFF);
						fSP0[i] -> SetEnabled(false);
						fSP1[i] -> SetEnabled(false);
						fSP0[i] -> SetValue(B_CONTROL_OFF);
						fSP1[i] -> SetValue(B_CONTROL_OFF);
					}
				}
			}
			break;

		case IE_MAINWINDOW_READIMMEDIATE:	// 'ReadImmediate' is pressed...
			{
			}
			break;


		default:
			inherited::MessageReceived(message);
			break;
	}

}

void MainWindow :: InitInterface()
{
	BTextView	*child;
	int i;
	
	fVendorIDControl = (BTextControl *) FindView("VendorID"); 
	child = (BTextView *)fVendorIDControl-> ChildAt(0);
	child -> SetMaxBytes(4);																// max. 4 bytes	
	for (i = 0; i <= 255; i++) child -> DisallowChar(i);		// exclude everyting
	for (i = 0; i <= 9; i++)   child -> AllowChar(48+i);		// only numbers  0..9
	
	fProductIDControl			= (BTextControl *) FindView("ProductID"); 
	child = (BTextView *)fProductIDControl-> ChildAt(0);
	child -> SetMaxBytes(4);																// max. 4 bytes	
	for (i = 0; i <= 255; i++) child -> DisallowChar(i);		// exclude everyting
	for (i = 0; i <= 9; i++)   child -> AllowChar(48+i);		// only numbers 0..9
	
	fSerialNumberControl 	= (BTextControl *) FindView("SerialNumber"); 
	child = (BTextView *)fSerialNumberControl-> ChildAt(0);
	child -> SetMaxBytes(8);																// max. 8 bytes	
	for (i = 0; i <= 255; i++) child -> DisallowChar(i);		// exclude everyting
	for (i = 0; i < 10; i++)   child -> AllowChar(0x30+i);	// numbers 0..9
	for (i = 0; i < 26; i++)   child -> AllowChar(0x41+i);	// capital characters A,B,..,Z
	//for (i = 0; i < 26; i++)   child -> AllowChar(0x61+i);	// a,b,..,c
	
	fMainView 			= (BView *) FindView("MainView" );
	fUSBDeviceBox		=	(BBox  *) FindView("USBDeviceBox");
	fSearchButton		= (BButton *) FindView("SearchButton");

	fDeviceBox			= (BBox *) FindView("DeviceBox");
	fDeviceBox			-> SetViewColor(255,255,255,255);
	
	fDeviceString		= (BStringView *) FindView("DeviceString");
	fDeviceString 	-> SetViewColor(255,255,255,255);
	
	fModeBox	= (BBox *) FindView("ModeBox");
	fStartButton	= (BButton *) FindView("StartButton");
	fInOutButton	= (BPictureButton *) FindView("INOUTButton");
	
	fReadImmediate = (BCheckBox *) FindView("ReadImmediate");
	
	fRedText1 = (BStringView *) FindView("RedText1");
	fRedText2 = (BStringView *) FindView("RedText2");
	
	fRedText1 -> SetViewColor(255,140,140,255);
	fRedText2 -> SetViewColor(255,140,140,255);
	
	char	setViewName[6]  = "SP0_";
	char	readViewName[6] = "RP0_";

	setViewName[2] 	= '0';														// SP0
	readViewName[2] = '0';														// RP0
	for (i=0; i<8; i++) 															// Port 0
	{
		setViewName[4] 	= 48+i;													// index 0..7
		readViewName[4] = 48+i;													// index 0..7
		fSP0[i]		= (BCheckBox *)FindView(setViewName);	
		fRP0[i]		= (BPictureButton *)FindView(readViewName);	
	}
	
	setViewName[2] 	= '1';														// SP1
	readViewName[2] = '1';														// RP1
	for (i=0; i<8; i++) 															// Port 1
	{
		setViewName[4] 	= 48+i;													// index 0..7
		readViewName[4] = 48+i;													// index 0..7
		fSP1[i]		= (BCheckBox *)FindView(setViewName);	
		fRP1[i]		= (BPictureButton *)FindView(readViewName);	
	}			
}


void MainWindow :: EnableAllElements(bool enable)
{
	// alle IF-Elemente aktivieren/deaktivieren
	
	int i;
	for (i=0; i<8; i++)
	{
		fSP0[i] -> SetEnabled(enable);
		fSP1[i] -> SetEnabled(enable);
		fSP0[i] -> SetValue(B_CONTROL_OFF);
		fSP1[i] -> SetValue(B_CONTROL_OFF);
	}
	
	fStartButton -> SetEnabled(enable);
	fInOutButton -> SetEnabled(enable);
	fReadImmediate -> SetEnabled(enable);
	fStartButton -> SetLabel(" Start Reading");
	ClearDisplay();
}


void MainWindow :: WriteDisplay(uint16 data)
{
	if (!fReceiving) return;
	
	Lock();
		uint16	wert;
		int			i;
		int 		pow2;
		
		wert = data & 0x00FF;			//Port 0
		pow2 = 128;
		for (i=7; i>=0; i--)
		{
			if (wert/pow2)
			{
				fRP0[i] -> SetValue(B_CONTROL_OFF);
				wert = wert - pow2;			
			}
			else fRP0[i] -> SetValue(B_CONTROL_ON);
			pow2 = pow2/2;
		}
	
		wert = data & 0xFF00;			//Port 1
		wert = wert >> 8;
		pow2 = 128;
		for (i=7; i>=0; i--)
		{
			if (wert/pow2)
			{
				fRP1[i] -> SetValue(B_CONTROL_OFF);
				wert = wert - pow2;			
			}
			else fRP1[i] -> SetValue(B_CONTROL_ON);
			pow2 = pow2/2;
		}
		
	Unlock();
}


void MainWindow :: ClearDisplay()
{
	for (int i=0; i<8; i++)
	{
		fRP0[i] -> SetValue(B_CONTROL_OFF);
		fRP1[i] -> SetValue(B_CONTROL_OFF);
	}
}


bool MainWindow :: ThreadRunning()
{
	return (fReceiving);
}


void MainWindow :: WriteModeBoxLabel(const char *label)
{
	Lock();
		fModeBox -> SetLabel(label);
	Unlock();
}


void MainWindow :: SetPrefs()
{
	fPrefs -> SetRect ("WindowFrame", Frame() );
	fPrefs -> SetInt32 ("VendorID",fVendorID);
	fPrefs -> SetInt32 ("ProductID",fProductID);
	fPrefs -> SetString ("SerialNumber",fSerialNumber);
//	fPrefs->PrintToStream();
}


void MainWindow  :: GetPrefs()
{
	status_t	err;
	char textField[20];

	fPrefs = new TPreferences ("IO-Warrior/IO16_Prefs");				// name pf prefs file
	if (fPrefs -> InitCheck() != B_OK)													// if no file -> create one
	{
		fWindFrame = Frame();																			// get window frame
		fPrefs -> SetRect ("WindowFrame", fWindFrame );		
	
		fVendorID = 0;
		fPrefs -> SetInt32 ("VendorID",fVendorID);

		fProductID = 0;
		fPrefs -> SetInt32 ("ProductID",fProductID);

		strcpy(fSerialNumber,"0");
		fPrefs -> SetString ("SerialNumber",fSerialNumber);
	}
	
	err = (fPrefs -> FindRect ("WindowFrame", &fWindFrame) );
	if (err == B_OK)
	{
		ResizeTo(fWindFrame.Width(), fWindFrame.Height() );
		MoveTo(fWindFrame.left, fWindFrame.top);												// from preferences		 
	} 
	err = (fPrefs -> FindInt32 ("VendorID",&fVendorID) );
	sprintf((char*)textField,"%d",(int)fVendorID);										// long to string
	fVendorIDControl ->SetText(textField); 														// write string
	
	err = (fPrefs -> FindInt32 ("ProductID",&fProductID) );
	sprintf((char*)textField,"%d",(int)fProductID);										// long to string
	fProductIDControl ->SetText(textField); 													// write string
	
	const char *string;
	err = (fPrefs -> FindString ("SerialNumber",&string) );
	strcpy(fSerialNumber,string);
	fSerialNumberControl ->SetText(fSerialNumber); 										// write string

}



